##************************************************************
##  Html/peppermint.coffee
##  Peppermint
##
##  Coded by Dr.Kameleon
##
##  Copyright (c) 2011-2014 InSili.co.
##  All rights reserved.
##************************************************************

root = exports ? this

###
@class Dialog
@description The Dialog object - used to invoke dialogs
###

class DialogObject

	constructor: ()->
		@callbacks = {};

	###
	@method alert
	@description Display an alert dialog
	@param title|required|string|The main dialog title
	@param information|required|string|Related information
	@param callback|optional|string|The function to be called, when the dialog closes
	@example $$Dialog.alert("Finished!","The operation has finished")
	###

	alert: (title,information,callback=null)->
		if callback?
			if callback instanceof String
				callback = "_plugin.#{callback}();";
			else
				if callback not in @callbacks
				    @callbacks.push(callback);
				ind = @callbacks.indexOf(callback);
				callback = "_plugin.Dialog.callbacks['#{guid}']();";

		core.alertDialogWithTitle_information_callback_(title,information,callback);

	###
	@method getInput
	@description Show an input dialog and get user input
	@param title|required|string|The title/prompt of the dialog
	@param callback|required|string|The function to be called, with the input, when the dialog closes
	@return null|
	@example my_plugin = {
	@example 	init: ()->
	@example 		$$Dialog.getInput("Search","gotInput");
	@example  
	@example 	gotInput: (input)->
	@example 		$$Terminal.echo("The input is: " + input);
	@example }
	###

	getInput: (title,callback)->
		result = _plugin.addCallback(callback);
		core.input_callback_(title, result);
		null

###
@class Document
@description The current Document object
###

class DocumentObject

	constructor: ()->

	###
	@method getContent
	@description Get current document's content
	@return string|The document's contents
	@example content = $$Document.getContent()
	###
	getContent: ()->
		return pp.getContent();

	###
	@method getLabel
	@description Get current document's label, as it appears in Peppermint
	@return string|The document's label
	@example label = $$Document.getLabel()
	###
	getLabel: ()->
		return core.getLabel();

	###
	@method getInfo
	@description Get DocumentInfo of current document
	@return DocumentInfo|A <a href='#api-class-documentinfo'>DocumentInfo</a> object or <code>null</code>, if document is not stored to disk
	@example documentInfo = $$Document.getInfo()
	###
	getInfo: ()->
		fullPath = pp.getFullPath();
		if fullPath==""
			return null
		else
			return new DocumentInfoObject(fullPath);

	###
	@method getTemp
	@description Get a temporary file for current document
	@param extension|required|string|Desired extension for the file
	@return DocumentInfo|A <a href='#api-class-documentonfo'>DocumentInfo</a> object with the details of the temporary file
	@example tempInfo = $$Document.getTemp()
	###
	getTemp: (extension)->
		filePath = core.getTempFile_withExtension_(@getContent(), extension);

		return new DocumentInfoObject(filePath);
		
	write: (path,contents)->
	    core.document_write_(path,contents);

###
@class DocumentInfo
@description An object representing current document's details
@property path|string|The full path of the document
@property filename|string|The document's pure filename
@property folder|string|The document's pure folder name
@property extension|string|The document's file extension
###

class DocumentInfoObject

	constructor: (filepath)->

		fileName = filepath.replace(/^.*[\\\/]/, '');
		folderPath = filepath.replace(fileName, '');

		re = /(?:\.([^.]+))?$/;

		extension = re.exec(fileName)[1];

		@path = filepath
		@filename = fileName;
		@folder = folderPath;
		@extension = extension;

###
@class On
@description The main Event-handling object
###

class OnObject

	@pluginName

	constructor: ()->
		
	###
	@method documentChanged
	@description Set callback to be execute, when document changes
	@param callback|required|function|The callback to be executed, or null to remove handler
	@return null|
	@example init: ()->
	@example    $$x = 1;
	@example    $$On.documentChanged(()=>
	@example        $$Terminal.echo("Document changed");
	@example        $$Terminal.echo("x = " + $$x++);
	@example    );
	@example  
	@example halt: ()->
	@example    $$On.documentChanged(null);
	###
	
	documentChanged: (callback)->
		if callback?
			window.__.documentChangedEvents[@pluginName] = callback;
		else
			delete window.__.documentChangedEvents[@pluginName];
		
	###
	@method cursorChanged
	@description Set callback to be execute, when document changes
	@param callback|required|function|The callback to be executed, or null to remove handler
	@return null|
	@example $$On.cursorChanged(()=>
	@example    # do something
	@example )
	###
		
	cursorChanged: (callback)->
		if callback?
			window.__.cursorChangedEvents[@pluginName] = callback;
		else
			delete window.__.cursorChangedEvents[@pluginName];


###
@class Terminal
@description The main Terminal object - representing the console panel
###

class TerminalObject

	constructor: ()->
		@environment = {}
		
	###
	@method show
	@description Show the terminal panel, if not already open
	@return null|
	@example $$Terminal.show()
	###
	show: ()->
		pp.showConsoleTab();
		null

	###
	@method clear
	@description Clear the terminal panel
	@return null|
	@example $$Terminal.clear()
	###
	clear: ()->
		terminal.clear();
		null

	###
	@method echo
	@description Echoes given string to the terminal
	@param message|required|string|The message to be displayed
	@param attributes|optional|string|A style attribute string
	@return null|
	@example $$Terminal.echo("Hello world!")
	###
	echo: (message,attributes=null)->
		msg = message;
		if attributes?
			msg = "[[#{attributes}]#{message}]";

		terminal.echo(msg);
		null

	###
	@method exec
	@description Execute command in terminal
	@param command|required|string|Command to be executed
	@param args|optional|string,...|Arguments to be pass
	@return string|The execution result
	@example $$Terminal.exec("php","version")
	###
	exec: (command,args...)->
		if args? and args[0] instanceof Array
			args = args[0]

		if (Object.keys(@environment).length==0)
			return pp.exec([ command ].concat(args), false);
		else
			return pp.execWithEnv([ command ].concat(args), @environment, false);

	setEnvironment: (variable,value)->
		@environment[variable] = value;


###
@class Utility
@description The main Utility object - representing the Utility Panel on the right side of the Peppermint window
###

class UtilityObject

	constructor: ()->
		
	###
	@method load
	@description Load html tool in the Utility View
	@param title|required|string|The new title for the Utility View
	@param path|required|string|The path to the .html file to be loaded
	@return null|
	@example $$Utility.load("My great tool", "index.html")
	###
	load: (title,path)->
		core.loadUtility_label_(path, title);
		null

	###
	@method loadDocument
	@description Load custom document in the Utility View
	@param title|required|string|The new title for the Utility View
	@param content|required|string|The script's contents
	@param spec|required|string|The spec to be used for highlighting
	@return null|
	@example $$Utility.loadDocument("My PHP Script", "<?php echo 'done'; ?>", "php")
	###

	loadDocument: (title,content,spec)->
		core.loadEditorUtility_label_mode_(content, title, spec);

class APIObject

	constructor: ()->
		
		@documentChangedEvents = [];
		@cursorChangedEvents = [];
		@Dialog = new DialogObject;
		@Document = new DocumentObject;
		@Terminal = new TerminalObject;
		@Utility = new UtilityObject;

	documentChanged: ()->
		for plugin,e of @documentChangedEvents
			e()

	cursorChanged: ()->
		for plugin,e of @cursorChangedEvents
			e()

	coffee: (code,bare=false,exec=false)->
	    if typeof code == 'object'
            result = CoffeeScript.compile(eval(code)[0],{ bare: bare });
        else 
            result = CoffeeScript.compile(code,{ bare: bare });
            
        if exec
            eval(result);
        else
            return result;
            
    haltRunningPlugin: ()->
        if window._plugin? and window._plugin.halt?
            window._plugin.halt();        
		
    initPlugin: (pluginName,pluginPath)->
        @haltRunningPlugin();
        if @[pluginName]? and window._plugin? and pluginName==window._plugin.name
            __.Terminal.echo("Reusing");
            window._plugin.init();
        else
            __.Terminal.echo("Creating");
            window._plugin = null;
            window._plugin =@[pluginName];
            window._plugin.callbacks = {};
            window._plugin.addCallback = (callback)->
                ind = pp.uuid();
                @callbacks["#{ ind }"] = callback;

                return "window._plugin.callbacks['#{ind}']('%@');";
                
            window._plugin.name = pluginName;
            window._plugin.path = pluginPath;
            window._plugin.Dialog = window.__.Dialog;
            window._plugin.Document = window.__.Document;
            window._plugin.On = new OnObject(pluginName);
            window._plugin.Terminal = new TerminalObject;
            window._plugin.Utility = window.__.Utility;
            window._plugin.inspect = (objs...)->
            	res = "";
            	for obj in objs
            		res += window.inspectObject(obj)

            	return res

            window._plugin.import = (path,execute=true)->
            	if execute
            		return core.importjs_(path);
            	else
            		return core.file_(path);
        
            if window._plugin.menu?
           	    core.registerPluginMenu_(JSON.stringify(Object.keys(window._plugin.menu())));
        
            window._plugin.init();

unless root.__
	root.__ = new APIObject;

window.runPlugin = (plContent,plName,plPath)->
	__.coffee(plContent,false,true);
	__.initPlugin(plName, plPath);

